@@ -14,10 +14,11 @@
  */
 /* Multibyte support	Eiji Tokuya 2001-03-15 */
 
+#include "loadlib.h"
+#include "connection.h"
 #ifndef	NOT_USE_LIBPQ
 #include <libpq-fe.h>
 #endif /* NOT_USE_LIBPQ */
-#include "connection.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -307,6 +308,7 @@ CC_Constructor()
 	{
 		rv->status = CONN_NOT_CONNECTED;
 		rv->transact_status = CONN_IN_AUTOCOMMIT;		/* autocommit by default */
+		rv->stmt_in_extquery = NULL;
 
 		CC_conninfo_init(&(rv->connInfo));
 		rv->sock = SOCK_Constructor(rv);
@@ -601,6 +603,7 @@ CC_cleanup(ConnectionClass *self)
 
 	self->status = CONN_NOT_CONNECTED;
 	self->transact_status = CONN_IN_AUTOCOMMIT;
+	self->stmt_in_extquery = NULL;
 	CC_conninfo_init(&(self->connInfo));
 	if (self->original_client_encoding)
 	{
@@ -765,6 +768,7 @@ EatReadyForQuery(ConnectionClass *conn)
 				CC_set_in_error_trans(conn);
 				break;	
 		}
+		conn->stmt_in_extquery = NULL;
 	}
 	return id;	
 }
@@ -1195,6 +1199,7 @@ static int	protocol3_packet_build(ConnectionClass *self)
 	return 1;
 }
 
+#ifndef	NOT_USE_LIBPQ
 CSTR	l_login_timeout = "connect_timeout";
 static char	*protocol3_opts_build(ConnectionClass *self)
 {
@@ -1264,6 +1269,7 @@ static char	*protocol3_opts_build(ConnectionClass *self)
 inolog("return conninfo=%s(%d)\n", conninfo, strlen(conninfo));
 	return conninfo;
 }
+#endif /* NOT_USE_LIBPQ */
 
 static char CC_initial_log(ConnectionClass *self, const char *func)
 {
@@ -2412,6 +2418,15 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 		return NULL;
 	}
 
+	/* Finish the pending extended query first */
+	if (!SyncParseRequest(self))
+	{
+		if (CC_get_errornumber(self) <= 0)
+		{
+			CC_set_error(self, CONN_EXEC_ERROR, "error occured while calling SyncParseRequest() in CC_send_query_append()", func);
+			return NULL;
+		}
+	}	
 	/* Indicate that we are sending a query to the backend */
 	maxlen = CC_get_max_query_len(self);
 	qrylen = strlen(query);
@@ -2789,12 +2804,11 @@ inolog("Discarded the first SAVEPOINT\n");
 					if (!QR_fetch_tuples(res, self, cursor))
 					{
 						CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, QR_get_message(res), func);
-						ReadyToReturn = TRUE;
 						if (PORES_FATAL_ERROR == QR_get_rstatus(res))
 							retres = cmdres;
 						else
 							retres = NULL;
-						break;
+						aborted = TRUE;
 					}
 					query_completed = TRUE;
 				}
@@ -2993,6 +3007,15 @@ CC_send_function(ConnectionClass *self, int fnid, void *result_buf, int *actual_
 		return FALSE;
 	}
 
+	/* Finish the pending extended query first */
+	if (!SyncParseRequest(self))
+	{
+		if (CC_get_errornumber(self) <= 0)
+		{
+			CC_set_error(self, CONN_EXEC_ERROR, "error occured while calling SyncParseRequest() in CC_send_function()", func);
+			return FALSE;
+		}
+	}	
 #define	return DONT_CALL_RETURN_FROM_HERE???
 	ENTER_INNER_CONN_CS(self, func_cs_count);
 	ci = &(self->connInfo);
@@ -3712,6 +3735,7 @@ inolog("status=%d\n", pqret);
 		{
 			mylog("password retry\n");
 			PQfinish(pqconn);
+			sock->pqconn = NULL;
 			self->sock = sock;
 			return -1;
 		}
