@@ -129,6 +129,40 @@ UInt4	add_removeExtraOptions(ConnInfo *ci, UInt4 aflag, UInt4 dflag)
 	return (ci->extra_opts = getExtraOptions(ci));
 }
 
+static const char *
+abbrev_sslmode(const char *sslmode, char *abbrevmode)
+{
+	switch (sslmode[0])
+	{
+		case SSLLBYTE_DISABLE:
+		case SSLLBYTE_ALLOW:
+		case SSLLBYTE_PREFER:
+		case SSLLBYTE_REQUIRE:
+			abbrevmode[0] = sslmode[0];
+			abbrevmode[1] = '\0';
+			break;
+		case SSLLBYTE_VERIFY:
+			abbrevmode[0] = sslmode[0];
+			abbrevmode[2] = '\0';
+			switch (sslmode[1])
+			{
+				case 'f':
+				case 'c':
+					abbrevmode[1] = sslmode[1];
+					break;
+				default:
+					if (strnicmp(sslmode, "verify_", 7) == 0)
+					{
+						abbrevmode[1] = sslmode[7];
+					}
+					else
+						strcpy(abbrevmode, sslmode);	
+			}
+			break;
+	} 
+	return abbrevmode;
+}
+
 void
 makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 {
@@ -169,7 +203,7 @@ inolog("hlen=%d", hlen);
 		if (ci->rollback_on_error >= 0)
 			snprintf(protocol_and, sizeof(protocol_and), "%s-%d", ci->protocol, ci->rollback_on_error);
 		else
-			strncpy(protocol_and, ci->protocol, sizeof(protocol_and));
+			strncpy_null(protocol_and, ci->protocol, sizeof(protocol_and));
 		olen = snprintf(&connect_string[hlen], nlen, ";"
 			INI_SSLMODE "=%s;"
 			INI_READONLY "=%s;"
@@ -310,8 +344,12 @@ inolog("hlen=%d", hlen);
 			flag |= BIT_LOWERCASEIDENTIFIER;
 
 		if (ci->sslmode[0])
+		{
+			char	abbrevmode[sizeof(ci->sslmode)];
+
 			olen = snprintf(&connect_string[hlen], nlen, ";"
-				ABBR_SSLMODE "=%c", ci->sslmode[0]);
+				ABBR_SSLMODE "=%s", abbrev_sslmode(ci->sslmode, abbrevmode));
+		}
 		hlen = strlen(connect_string);
 		nlen = MAX_CONNECT_STRING - hlen;
 		olen = snprintf(&connect_string[hlen], nlen, ";"
@@ -490,8 +528,17 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 
 	else if (stricmp(attribute, INI_CONNSETTINGS) == 0 || stricmp(attribute, ABBR_CONNSETTINGS) == 0)
 	{
-		decode(value, ci->conn_settings);
-		/* strcpy(ci->conn_settings, value); */
+		if ('{' == *value)
+		{
+			size_t	len;
+
+			strncpy_null(ci->conn_settings, value + 1, sizeof(ci->conn_settings));
+			len = strlen(ci->conn_settings);
+			if (len > 0 && '}' == ci->conn_settings[len - 1])
+				ci->conn_settings[len - 1] = '\0';
+		}
+		else
+			decode(value, ci->conn_settings);
 	}
 	else if (stricmp(attribute, INI_DISALLOWPREMATURE) == 0 || stricmp(attribute, ABBR_DISALLOWPREMATURE) == 0)
 		ci->disallow_premature = atoi(value);
@@ -513,16 +560,29 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 	{
 		switch (value[0])
 		{
-			case 'a':
+			case SSLLBYTE_ALLOW:
 				strcpy(ci->sslmode, SSLMODE_ALLOW);
 				break;
-			case 'p':
+			case SSLLBYTE_PREFER:
 				strcpy(ci->sslmode, SSLMODE_PREFER);
 				break;
-			case 'r':
+			case SSLLBYTE_REQUIRE:
 				strcpy(ci->sslmode, SSLMODE_REQUIRE);
 				break;
-			case 'd':
+			case SSLLBYTE_VERIFY:
+				switch (value[1])
+				{
+					case 'f':
+						strcpy(ci->sslmode, SSLMODE_VERIFY_FULL);
+						break;
+					case 'c':
+						strcpy(ci->sslmode, SSLMODE_VERIFY_CA);
+						break;
+					default:
+						strcpy(ci->sslmode, value);
+				}
+				break;
+			case SSLLBYTE_DISABLE:
 			default:
 				strcpy(ci->sslmode, SSLMODE_DISABLE);
 				break;
@@ -697,6 +757,18 @@ getDriverNameFromDSN(const char *dsn, char *driver_name, int namelen)
 	return SQLGetPrivateProfileString(ODBC_DATASOURCES, dsn, "", driver_name, namelen, ODBC_INI);
 }
 
+int
+getLogDir(char *dir, int dirmax)
+{
+	return SQLGetPrivateProfileString(DBMS_NAME, INI_LOGDIR, "", dir, dirmax, ODBCINST_INI);
+}
+
+int
+setLogDir(const char *dir)
+{
+	return SQLWritePrivateProfileString(DBMS_NAME, INI_LOGDIR, dir, ODBCINST_INI);
+}
+
 void
 getDSNinfo(ConnInfo *ci, char overwrite)
 {
@@ -1072,7 +1144,7 @@ writeDSNinfo(const ConnInfo *ci)
 	if (ci->rollback_on_error >= 0)
 		sprintf(temp, "%s-%d", ci->protocol, ci->rollback_on_error);
 	else
-		strncpy(temp, ci->protocol, sizeof(temp));
+		strncpy_null(temp, ci->protocol, sizeof(temp));
 	SQLWritePrivateProfileString(DSN,
 								 INI_PROTOCOL,
 								 temp,
