@@ -21,6 +21,7 @@
 extern "C" {
 #endif /* __cplusplus */
 /*	Unknown data type sizes */
+#define UNKNOWNS_AS_DEFAULT			-1
 #define UNKNOWNS_AS_MAX				0
 #define UNKNOWNS_AS_DONTKNOW			1
 #define UNKNOWNS_AS_LONGEST			2
@@ -142,11 +143,19 @@ extern "C" {
 #define INI_SSLMODE			"SSLmode"
 #define ABBR_SSLMODE			"CA"
 #define INI_EXTRAOPTIONS		"AB"
+#define INI_LOGDIR			"Logdir"
 
 #define	SSLMODE_DISABLE		"disable"
 #define	SSLMODE_ALLOW		"allow"
 #define	SSLMODE_PREFER		"prefer"
 #define	SSLMODE_REQUIRE		"require"
+#define	SSLMODE_VERIFY_CA	"verify-ca"
+#define	SSLMODE_VERIFY_FULL	"verify-full"
+#define	SSLLBYTE_DISABLE	'd'
+#define	SSLLBYTE_ALLOW		'a'
+#define	SSLLBYTE_PREFER		'p'
+#define	SSLLBYTE_REQUIRE	'r'
+#define	SSLLBYTE_VERIFY		'v'
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 #define INI_XAOPT			"XaOpt"
@@ -276,6 +285,8 @@ void		makeConnectString(char *connect_string, const ConnInfo *ci, UWORD);
 BOOL		copyAttributes(ConnInfo *ci, const char *attribute, const char *value);
 BOOL		copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value);
 int	getDriverNameFromDSN(const char *dsn, char *driver_name, int namelen);
+int	getLogDir(char *dir, int dirmax);
+int	setLogDir(const char *dir);
 int     changeDriverNameOfaDSN(const char *dsn, const char *driver_name, DWORD *errcode);
 UInt4	getExtraOptions(const ConnInfo *);
 BOOL	setExtraOptions(ConnInfo *, const char *str, const char *format);
