@@ -27,6 +27,8 @@
 #define IDS_SSLREQUEST_REQUIRE		412
 #define IDS_SSLREQUEST_DISABLE		413
 #define IDC_NOTICE_USER			414
+#define IDS_SSLREQUEST_VERIFY_CA	415
+#define IDS_SSLREQUEST_VERIFY_FULL	416
 #define DLG_CONFIG                      1001
 #define IDC_PORT                        1002
 #define DLG_DRIVER_CHANGE               1002
@@ -94,6 +96,7 @@
 #define DRV_DTCLOG			1083
 #define DS_EXTRA_OPTIONS		1084
 #define IDC_TEST			1085
+#define DS_LOGDIR			1086
 
 // Next default values for new objects
 // 
@@ -101,7 +104,7 @@
 #ifndef APSTUDIO_READONLY_SYMBOLS
 #define _APS_NEXT_RESOURCE_VALUE        105
 #define _APS_NEXT_COMMAND_VALUE         40001
-#define _APS_NEXT_CONTROL_VALUE         1086
+#define _APS_NEXT_CONTROL_VALUE         1087
 #define _APS_NEXT_SYMED_VALUE           101
 #endif
 #endif
